
from wxPython.wx       import *
from wxPython.glcanvas import *
from OpenGL.GL import *
from OpenGL.GLUT import *
import _cmd
import threading
import pymol

P_GLUT_IDLE_EVENT          =  0
P_GLUT_DISPLAY_EVENT       =  1
P_GLUT_RESHAPE_EVENT       =  2
P_GLUT_MOUSE_EVENT         =  3
P_GLUT_MOTION_EVENT        =  4
P_GLUT_CHAR_EVENT          =  5

P_GLUT_ACTIVE_ALT             =  32
P_GLUT_ACTIVE_CTRL            =  64
P_GLUT_ACTIVE_SHIFT           =  128

P_GLUT_DOWN                   =  9
P_GLUT_UP                     =  10
P_GLUT_KEY_DOWN               =  11
P_GLUT_KEY_LEFT               =  12
P_GLUT_KEY_RIGHT              =  13
P_GLUT_KEY_UP                 =  14
P_GLUT_LEFT_BUTTON            =  15
P_GLUT_MIDDLE_BUTTON          =  16
P_GLUT_RIGHT_BUTTON           =  17

if 1:

    class MyCanvasBase(wxGLCanvas):
        def __init__(self, parent):
            wxGLCanvas.__init__(self, parent, -1)
            self.init = false
            # initial mouse position
            self.lastx = self.x = 30
            self.lasty = self.y = 30
            EVT_ERASE_BACKGROUND(self, self.OnEraseBackground)
            EVT_SIZE(self, self.OnSize)
            EVT_PAINT(self, self.OnPaint)
            EVT_LEFT_DOWN(self, self.OnMouseDown)  
            EVT_LEFT_UP(self, self.OnMouseUp)
            EVT_MIDDLE_DOWN(self, self.OnMouseDown)
            EVT_MIDDLE_UP(self, self.OnMouseUp)
            EVT_RIGHT_DOWN(self, self.OnMouseDown) 
            EVT_RIGHT_UP(self, self.OnMouseUp)
            EVT_MOTION(self, self.OnMouseMotion)
            EVT_IDLE(self,self.OnIdle)
            EVT_CHAR(self,self.OnChar)
            self.mouse = ''
            self.mod = 0
            
        def OnEraseBackground(self, event):
            pass # Do nothing, to avoid flashing on MSW.

        def OnSize(self, event):
            size = self.GetClientSize()
            if self.GetContext():
                self.SetCurrent()
                glViewport(0, 0, size.width, size.height)
                _cmd.p_glut_event(P_GLUT_RESHAPE_EVENT,size.width,size.height,0,0,0)

        def OnChar(self, evt):
            self.mod = 0
            if evt.ShiftDown():
                self.mod = self.mod + P_GLUT_ACTIVE_SHIFT
            if evt.ControlDown():
                self.mod = self.mod + P_GLUT_ACTIVE_CTRL
            if evt.MetaDown():
                self.mod = self.mod + P_GLUT_ACTIVE_ALT
            _cmd.p_glut_event(P_GLUT_CHAR_EVENT,evt.GetX(),evt.GetY(),evt.GetKeyCode(),0,self.mod)
                              
        def OnPaint(self, event):
            dc = wxPaintDC(self)
            self.SetCurrent()
            if not self.init:
                self.InitGL()
                self.init = true
            self.OnDraw()

        def OnMouseDown(self, evt):
            self.CaptureMouse()
            x,y = evt.GetPosition()
            self.mod = 0
            if evt.ShiftDown():
                self.mod = self.mod + P_GLUT_ACTIVE_SHIFT
            if evt.ControlDown():
                self.mod = self.mod + P_GLUT_ACTIVE_CTRL
            if evt.LeftIsDown():
                _cmd.p_glut_event(P_GLUT_MOUSE_EVENT,x,y,P_GLUT_LEFT_BUTTON,P_GLUT_DOWN,self.mod)
                self.Repaint()
                self.mouse='l'
            elif evt.MiddleIsDown():
                _cmd.p_glut_event(P_GLUT_MOUSE_EVENT,x,y,P_GLUT_MIDDLE_BUTTON,P_GLUT_DOWN,self.mod)
                self.mouse='m'
                self.Repaint()                
            elif evt.RightIsDown():
                _cmd.p_glut_event(P_GLUT_MOUSE_EVENT,x,y,P_GLUT_RIGHT_BUTTON,P_GLUT_DOWN,self.mod)
                self.mouse='r'
                self.Repaint()
            self.CheckPyMOL()                
#            print "mouse down"
            
        def OnMouseUp(self, evt):
            x,y = evt.GetPosition()
            if self.mouse=='l':
                _cmd.p_glut_event(P_GLUT_MOUSE_EVENT,x,y,P_GLUT_LEFT_BUTTON,P_GLUT_UP,self.mod)
                self.Repaint()
            elif self.mouse=='m':
                _cmd.p_glut_event(P_GLUT_MOUSE_EVENT,x,y,P_GLUT_MIDDLE_BUTTON,P_GLUT_UP,self.mod)
                self.Repaint()
            elif self.mouse=='r':
                _cmd.p_glut_event(P_GLUT_MOUSE_EVENT,x,y,P_GLUT_RIGHT_BUTTON,P_GLUT_UP,self.mod)
                self.Repaint()                
            self.mouse = ''
            self.CheckPyMOL()
#            print "mouse up"
            self.ReleaseMouse()
            
        def OnMouseMotion(self, evt):
            if evt.Dragging():
                x,y = evt.GetPosition()
                if evt.LeftIsDown():
                    _cmd.p_glut_event(P_GLUT_MOTION_EVENT,x,y,P_GLUT_LEFT_BUTTON,0,self.mod)
                    self.Repaint()
                elif evt.MiddleIsDown():
                    _cmd.p_glut_event(P_GLUT_MOTION_EVENT,x,y,P_GLUT_MIDDLE_BUTTON,0,self.mod)                
                    self.Repaint()
                elif evt.RightIsDown():
                    _cmd.p_glut_event(P_GLUT_MOTION_EVENT,x,y,P_GLUT_RIGHT_BUTTON,0,self.mod)
                    self.Repaint()
                self.CheckPyMOL()                
#                print "mouse motion"
            
        def OnIdle(self,evt):
            _cmd.p_glut_event(0,0,0,0,0,0) # idle event
#            self.AddPendingEvent(wxIdleEvent())
#            evt.RequestMore(true)
            self.CheckPyMOL()

        def Repaint(self):
            self.AddPendingEvent(wxPaintEvent())
            
    class CubeCanvas(MyCanvasBase):
        def InitGL(self):

            _cmd.runwxpymol()
            pymol._swap_buffers = lambda s=self: s.swap()
            _cmd.p_glut_event(P_GLUT_RESHAPE_EVENT,800,500,0,0,0) # initial reshape event
            self.CheckPyMOL()

        def swap(self):
            if self.GetContext():
                self.SetCurrent()
#                print "swapping"
                self.SwapBuffers()
            
        def OnDraw(self):
            _cmd.p_glut_event(P_GLUT_DISPLAY_EVENT,0,0,0,0,0) # draw event

        def CheckPyMOL(self):
            if _cmd.p_glut_get_redisplay():
                self.Repaint()

#----------------------------------------------------------------------

overview = """\
"""
#----------------------------------------------------------------------

ID_ABOUT=101  
ID_OPEN=102 
ID_BUTTON1=110 
ID_EXIT=200
        
class MainWindow(wxFrame):
   def __init__(self,parent,id,title):  
      self.dirname='' 
      wxFrame.__init__(self,parent,-4, title, style=wxDEFAULT_FRAME_STYLE|  
                              wxNO_FULL_REPAINT_ON_RESIZE)  
      self.control = wxTextCtrl(self, 1, style=wxTE_MULTILINE)  
      self.CreateStatusBar() # A Statusbar in the bottom of the window  
      # Setting up the menu.  
      filemenu= wxMenu()  
      filemenu.Append(ID_OPEN, "&Open"," Open a file to edit")  
      filemenu.AppendSeparator()  
      filemenu.Append(ID_ABOUT, "&About"," Information about this program")  
      filemenu.AppendSeparator()  
      filemenu.Append(ID_EXIT,"E&xit"," Terminate the program")  
      # Creating the menubar.  
      menuBar = wxMenuBar()  
      menuBar.Append(filemenu,"&File") # Adding the "filemenu" to the MenuBar  
      self.SetMenuBar(menuBar)  # Adding the MenuBar to the Frame content.  
      EVT_MENU(self, ID_ABOUT, self.OnAbout) 
      EVT_MENU(self, ID_EXIT, self.OnExit) 
      EVT_MENU(self, ID_OPEN, self.OnOpen) 

      self.sizer2 = wxBoxSizer(wxHORIZONTAL) 
      self.buttons=[] 
      for i in range(0,6): 
         self.buttons.append(wxButton(self, ID_BUTTON1+i, "Button &"+`i`)) 
         self.sizer2.Add(self.buttons[i],1,wxEXPAND) 

      self.canvas = CubeCanvas(self)

      # Use some sizers to see layout options 
      self.sizer=wxBoxSizer(wxVERTICAL) 
      self.sizer.Add(self.control,1,wxEXPAND) 
      self.sizer.Add(self.sizer2,0,wxEXPAND) 
      self.sizer.Add(self.canvas,1,wxEXPAND)
      
      #Layout sizers 
      self.SetSizer(self.sizer) 
      self.SetAutoLayout(1) 
      self.sizer.Fit(self) 

      self.Show(1)  

   def OnAbout(self,e):  
      d= wxMessageDialog( self, " A sample editor \n"  
                     " in wxPython","About Sample Editor", wxOK)  
                     # Create a message dialog box  
      d.ShowModal() # Shows it  
      d.Destroy() # finally destroy it when finished.  

   def OnExit(self,e):  
      self.Close(true)  # Close the frame.  

   def OnOpen(self,e):  
      """ Open a file""" 
      dlg = wxFileDialog(self, "Choose a file", self.dirname, "", "*.*", wxOPEN)  
      if dlg.ShowModal() == wxID_OK:  
         self.filename=dlg.GetFilename()  
         self.dirname=dlg.GetDirectory()  
         f=open(self.dirname+'\\'+self.filename,'r')  
         self.control.SetValue(f.read())  
         f.close()  
      dlg.Destroy()  
       
   
class MyApp(wxApp):
  def OnInit(self):
      mw = MainWindow(None, -1,"PyMOL")
      import threading
      return TRUE

#      frame = wxFrame(None, -1, "PyMOL", size=(1024,768))
#      frame.canvas = CubeCanvas(frame)
#      frame.control = wxTextCtrl(frame,1,style=wxTE_MULTILINE)
#      frame.sizer = wxBoxSizer(wxVERTICAL)
#      frame.sizer.Add(frame.canvas,1,wxEXPAND)
#      frame.sizer.Add(frame.control,1,wxEXPAND)
#      frame.sizer.Fit(frame)
#      frame.Show(true)
#      self.SetTopWindow(frame)

app = MyApp(0)
app.MainLoop()




#
#
#

from chempy.neighbor import Neighbor
from chempy.models import Connected
from chempy import Bond
from chempy import place

MAX_BOND_LEN = 2.2
PEPT_CUTOFF = 1.7

#---------------------------------------------------------------------------------
def generate(model, topology= None, forcefield = None ):
   
   add_bonds(model,topology=topology,forcefield=forcefield)
   connected = model.convert_to_connected()
   add_hydrogens(connected,topology=topology,forcefield=forcefield)
   place.simple_unknowns(connected)
   return connected.convert_to_indexed()

#---------------------------------------------------------------------------------
def assign_types(model, topology = None, forcefield = None ):
   if str(model.__class__) != 'chempy.models.Indexed':
      raise ValueError('model is not an "Indexed" model object')
   nAtom = model.nAtom
   if nAtom:
      tmpl = topology.normal
      ffld = forcefield.normal
      res_list = model.get_residues()
      if len(res_list):
         for a in res_list:
            base = model.atom[a[0]]
            resn = base.resn
            if not tmpl.has_key(resn):
               raise RuntimeError("unknown residue type '"+resn+"'")
            else:
               # reassign atom names and build dictionary
               dict = {}
               aliases = tmpl[resn]['aliases']
               for b in range(a[0],a[1]):
                  at = model.atom[b]
                  if aliases.has_key(at.name):
                     at.name = aliases[at.name]
                  dict[at.name] = b
                  if forcefield:
                     k = (resn,at.name)
                     if ffld.has_key(k):
                        at.text_type = ffld[k]['type']
                        at.partial_charge = ffld[k]['charge']
                     else:
                        raise RuntimeError("no parameters for '"+str(k)+"'")

#---------------------------------------------------------------------------------
def add_bonds(model, topology = None, forcefield = None ):
   if str(model.__class__) != 'chempy.models.Indexed':
      raise ValueError('model is not an "Indexed" model object')
   nAtom = model.nAtom
   if nAtom:
      tmpl = topology.normal
      ffld = forcefield.normal
      res_list = model.get_residues()
      if len(res_list):
         for a in res_list:
            base = model.atom[a[0]]
            resn = base.resn
            if not tmpl.has_key(resn):
               raise RuntimeError("unknown residue type '"+resn+"'")
            else:
               # reassign atom names and build dictionary
               dict = {}
               aliases = tmpl[resn]['aliases']
               for b in range(a[0],a[1]):
                  at = model.atom[b]
                  if aliases.has_key(at.name):
                     at.name = aliases[at.name]
                  dict[at.name] = b
                  if forcefield:
                     k = (resn,at.name)
                     if ffld.has_key(k):
                        at.text_type = ffld[k]['type']
                        at.partial_charge = ffld[k]['charge']
                     else:
                        raise RuntimeError("no parameters for '"+str(k)+"'")
               # now add bonds for atoms which are present
               bonds = tmpl[resn]['bonds']
               mbond = model.bond
               for b in bonds.keys():
                  if dict.has_key(b[0]) and dict.has_key(b[1]):
                     bnd = Bond()
                     bnd.index = [ dict[b[0]], dict[b[1]] ]
                     bnd.order = bonds[b]['order']
                     mbond.append(bnd)
                     
#---------------------------------------------------------------------------------
def add_hydrogens(model,topology=None,forcefield=None):  
   if str(model.__class__) != 'chempy.models.Connected':
      raise ValueError('model is not a "Connected" model object')
   nAtom = model.nAtom
   if nAtom:
      if not model.index:
         model.update_index()
      ffld = forcefield.normal
      tmpl = topology.normal
      res_list = model.get_residues()
      if len(res_list):
         for a in res_list:
            base = model.atom[a[0]]
            resn = base.resn
            if not tmpl.has_key(resn):
               raise RuntimeError("unknown residue type '"+resn+"'")
            else:
               # build dictionary
               dict = {}
               for b in range(a[0],a[1]):
                  at = model.atom[b]
                  dict[at.name] = b
               # find missing bonds with hydrogens
               bonds = tmpl[resn]['bonds']
               mbond = model.bond
               for b in bonds.keys():
                  if dict.has_key(b[0]) and (not dict.has_key(b[1])):
                     at = model.atom[dict[b[0]]]
                     if at.symbol != 'H':
                        name = b[1]
                        symbol = tmpl[resn]['atoms'][name]['symbol']
                        if symbol == 'H':
                           newat = at.new_in_residue()
                           newat.name = name
                           newat.symbol = symbol
                           k = (resn,newat.name)
                           newat.text_type = ffld[k]['type']
                           newat.partial_charge = ffld[k]['charge']
                           idx1 = model.index[id(at)]
                           idx2 = model.add_atom(newat)
                           bnd = Bond()
                           bnd.index = [ idx1, idx2 ]
                           bnd.order = bonds[b]['order']
                           mbond[idx1].append(bnd)
                           mbond[idx2].append(bnd)
                  if (not dict.has_key(b[0])) and dict.has_key(b[1]):
                     at = model.atom[dict[b[1]]]
                     if at.symbol != 'H':
                        name = b[0]
                        symbol = tmpl[resn]['atoms'][name]['symbol']
                        if symbol == 'H':
                           newat = at.new_in_residue()
                           newat.name = name
                           newat.symbol = symbol
                           k = (resn,newat.name)
                           newat.text_type = ffld[k]['type']
                           newat.partial_charge = ffld[k]['charge']
                           idx1 = model.index[id(at)]
                           idx2 = model.add_atom(newat)
                           bnd = Bond()
                           bnd.index = [ idx1, idx2 ]
                           bnd.order = bonds[b]['order']
                           mbond[idx1].append(bnd)
                           mbond[idx2].append(bnd)





from pymol.wizard import Wizard
from pymol import cmd
import pymol

import traceback

sele_prefix = "_dw"
sele_prefix_len = len(sele_prefix)

dist_prefix = "wdist"

default_mode = 'pairs'
default_object_mode  = 'overwr'
dist_count = 0

class Distance(Wizard):

   cutoff = 3.5
   
   def __init__(self):

      Wizard.__init__(self)
      
      self.status = 0 # 0 no atoms selections, 1 atom selected
      self.error = None
      self.object_name = None

      # mode selection subsystem
      
      self.mode = default_mode
      self.modes = [
         'polar',
         'heavy',
         'neigh',
         'pairs',
         ]
      self.mode_name = {
         'polar':'Polar Neighbors',
         'heavy':'Heavy Neighbors',
         'neigh':'Neighbors',
         'pairs':'Pairwise Distances',
         }

      smm = []
      smm.append([ 2, 'Measurement Mode', '' ])
      for a in self.modes:
         smm.append([ 1, self.mode_name[a], 'cmd.get_wizard().set_mode("'+a+'")'])
      self.menu['mode']=smm

      # overwrite mode selection subsystem
      
      self.object_mode=default_object_mode
      self.object_modes = [
         'overwr',
         'append',
         ]
      self.object_mode_name = {
         'overwr':'Overwrite',
         'append':'Append',         
         }

      smm = []
      smm.append([ 2, 'New Distances?', '' ])
      for a in self.object_modes:
         smm.append([ 1, self.object_mode_name[a], 'cmd.get_wizard().set_object_mode("'+a+'")'])
      self.menu['object_mode']=smm

# generic set routines

   def set_mode(self,mode):
      if mode in self.modes:
         self.mode = mode
      self.status = 0
      self.clear()
      cmd.refresh_wizard()

   def set_object_mode(self,mode):
      if mode in self.object_modes:
         self.object_mode = mode
      self.status = 0
      cmd.refresh_wizard()

      
   def get_panel(self):
      return [
         [ 1, 'Distance Measurement',''],
         [ 3, self.mode_name[self.mode],'mode'],
         [ 3, self.object_mode_name[self.object_mode],'object_mode'],
         [ 2, 'Delete Last' , 'cmd.get_wizard().delete_last()'],
         [ 2, 'Done','cmd.set_wizard()'],
         ]

   def cleanup(self):
      global default_mode, default_object_mode
      default_mode = self.mode
      default_object_mode = self.object_mode
      self.clear()
      
   def clear(self):
      cmd.delete(sele_prefix+"*")
      
   def get_prompt(self):
      self.prompt = None
      if self.mode == 'pairs':
         if self.status==0:
            self.prompt = [ 'Pick the first atom...']
         elif self.status==1:
            self.prompt = [ 'Pick the second atom...' ]
      elif self.mode in [ 'polar', 'neigh' ]:
         self.prompt = [ 'Pick an atom...']
      if self.error!=None:
         self.prompt.append(self.error)
      return self.prompt
   
   def delete_last(self):
      global dist_count
      if self.status==0:
         if dist_count>0:
            cmd.delete(dist_prefix+"%02d"%dist_count)
            dist_count = dist_count - 1
      self.status=0
      self.error = None
      self.clear()
      cmd.refresh_wizard()
      
   def do_pick(self,bondFlag):
      global dist_count

      if bondFlag:
         self.error = "Error: please select an atom, not a bond."
         print self.error
      else:
         if self.mode == 'pairs':
            if self.status==0:
               name = sele_prefix 
               cmd.select(name,"(pk1)")
               cmd.unpick()
               cmd.enable(name)
               self.status = 1
               self.error = None
            elif self.status==1:
               if ((self.object_mode=='append') or (not dist_count)):
                  dist_count = dist_count + 1
               else:
                  cmd.delete(dist_prefix+"%2d"%dist_count)
               name = dist_prefix + "%02d"%dist_count
               cmd.dist(name,sele_prefix,"(pk1)")
               cmd.delete(sele_prefix)
               cmd.unpick()
               cmd.enable(name)
               self.status = 0
         elif self.mode in ['neigh','polar','heavy']:
            if ((self.object_mode=='append') or (not dist_count)):
               dist_count = dist_count + 1
            else:
               cmd.delete(dist_prefix+"%2d"%dist_count)
            name = dist_prefix + "%02d"%dist_count
            cnt = 0
            if self.mode == 'neigh':
               cnt = cmd.select(sele_prefix,"(v. and (pk1 a; %f) and (not (nbr. pk1)) and (not (nbr. (nbr. pk1))) and (not (nbr. (nbr. (nbr. pk1)))))"%self.__class__.cutoff)
            elif self.mode == 'polar':
               cnt = cmd.select(sele_prefix,"(v. and (pk1 a; %f) and (e. n,o) and (not (nbr. pk1)) and (not (nbr. (nbr. pk1))) and (not (nbr. (nbr. (nbr. pk1)))))"%self.__class__.cutoff)            
            elif self.mode == 'heavy':
               cnt = cmd.select(sele_prefix,"(v. and (pk1 a; %f) and (not h.) and (not (nbr. pk1)) and (not (nbr. (nbr. pk1))) and (not (nbr. (nbr. (nbr. pk1)))))"%self.__class__.cutoff)            
            cmd.delete(name)
            if cnt:
               cmd.dist(name,"(pk1)",sele_prefix)
            else:
               print " Wizard: No neighbors found."
            cmd.delete(sele_prefix)
            cmd.unpick()
            cmd.enable(name)
      cmd.refresh_wizard()




import chempy

def get_partial_charge(model):
   if chempy.feedback['verbose']:
      print ' '+str(__name__)+': generating partial charge keywords...'
   list = []
   c = -1
   for a in model.atom:
      list.append("CHARGE %d %6.4f\n" %(c,a.partial_charge))
      c = c - 1
   return list


def get_restrain_positions(model,flag,w_width,f_cnst):
   list = []
   n = 0
   c = 1
   mask = 1<<flag
   for a in model.atom:
      if (a.flags&mask):
         list.append("RESTRAIN-POSITION %5d %12.6f %12.6f %12.6f %6.3f %6.1f\n" %
                     (c,a.coord[0],a.coord[1],a.coord[2],w_width,f_cnst))
         n = n + 1
      c = c + 1
   if chempy.feedback['actions']:
      print ' '+str(__name__)+': %d atoms restrained using flag %d ...' % (n,flag)
      
   return list

def get_inactive(model,flag):
   list = []
   n = 0
   c = 1
   mask = 1<<flag
   for a in model.atom:
      if (a.flags&mask):
         list.append("INACTIVE %d\n" % (c))
         n = n + 1
      c = c + 1
   if chempy.feedback['actions']:
      print ' '+str(__name__)+': %d atoms fixed using flag %d ...' % (n,flag)
   return list





from chempy.models import Indexed
from chempy import water_amber,water_residues
from chempy import hetatm,path,io
import os

import copy

def fill_box(area,water=None,box=18.774349):
   if not water:
      test_path = path + "water.pdb"
      if os.path.exists(test_path):
         water= io.pdb.fromFile(test_path)
         water= hetatm.generate(water,forcefield=water_amber,
                             topology=water_residues)
   if not water:
      raise RunError('Need water structure file.')
   solv_box = Indexed()
   half = box/2
   x = area[0][0]
   c = 0
   while x<area[1][0]:
      y = area[0][1]
      while y<area[1][1]:
         z = area[0][2]
         while z<area[1][2]:
            tmp = copy.deepcopy(water)
            lr = ''
            for a in tmp.atom:
               if a.resi!=lr:
                  c = c + 1
                  lr = a.resi
               a.resi = str(c)
               b = a.coord
               b[0] = b[0] + x + half
               b[1] = b[1] + y + half
               b[2] = b[2] + z + half
            print " "+__name__+": filling box at %8.3f %8.3f %8.3f\n" % (x,y,z)
            solv_box.merge(tmp)
            z = z + box
         y = y + box
      x = x + box
   return solv_box


